<?php
/*

// *** Include the class
include("resize-class.php");

// *** 1) Initialize / load image
$resizeObj = new resize('sample.jpg');

// *** 2) Resize image (options: exact, portrait, landscape, auto, crop)
$resizeObj -> resizeImage(150, 100, 'crop');

// *** 3) Save image
$resizeObj -> saveImage('sample-resized.gif', 100);

*/

class resize{

  private $image;
  private $width;
  private $height;
  private $imageResized;
  private $imageType;
  private $filename;

  function __construct($fileName){

      $this->image = $this->openImage($fileName);
      $this->filename=$fileName;
      $this->width  = imagesx($this->image);
      $this->height = imagesy($this->image);
  }

  private function getImageWHT($image){

    return getimagesize($image);

  }

  private function openImage($file){

    list($imagewidth, $imageheight, $imageType) = $this->getImageWHT($file);

    $imageType = image_type_to_mime_type($imageType);
    $this->imageType = $imageType;

    switch($this->imageType) {
      case "image/gif": $img=@imagecreatefromgif($file); break;
      case "image/pjpeg":
      case "image/jpeg":
      case "image/jpg": $img=@imagecreatefromjpeg($file); break;
      case "image/png":
      case "image/x-png": $img=@imagecreatefrompng($file); break;
      default: return false;
    }

    return $img;
  }

  public function resizeImage($newWidth, $newHeight, $option="auto", $expand = true){

    $optionArray = $this->getDimensions($newWidth, $newHeight, strtolower($option));

    $optimalWidth  = $optionArray['optimalWidth'];
    $optimalHeight = $optionArray['optimalHeight'];

    if ($expand || ($this->width > $optimalWidth || $this->height > $optimalHeight)) {

      $this->imageResized = imagecreatetruecolor($optimalWidth, $optimalHeight);

      //for transparent pngs
      //imagealphablending($this->imageResized, false);
      //imagesavealpha($this->imageResized, true);

      imagecopyresampled($this->imageResized, $this->image, 0, 0, 0, 0, $optimalWidth, $optimalHeight, $this->width, $this->height);

    }

    // *** if option is 'crop', then crop too
    if ($option == 'crop') {
        $this->crop($optimalWidth, $optimalHeight, $newWidth, $newHeight);
    }
  }

  private function getDimensions($newWidth, $newHeight, $option){

    switch ($option){
      case 'exact':
          $optimalWidth = $newWidth;
          $optimalHeight= $newHeight;
          break;
      case 'portrait':
          $optimalWidth = $this->getSizeByFixedHeight($newHeight);
          $optimalHeight= $newHeight;
          break;
      case 'landscape':
          $optimalWidth = $newWidth;
          $optimalHeight= $this->getSizeByFixedWidth($newWidth);
          break;
      case 'auto':
          $optionArray = $this->getSizeByAuto($newWidth, $newHeight);
          $optimalWidth = $optionArray['optimalWidth'];
          $optimalHeight = $optionArray['optimalHeight'];
          break;
      case 'crop':
          $optionArray = $this->getOptimalCrop($newWidth, $newHeight);
          $optimalWidth = $optionArray['optimalWidth'];
          $optimalHeight = $optionArray['optimalHeight'];
      break;
    }

    return array('optimalWidth' => $optimalWidth, 'optimalHeight' => $optimalHeight);

  }

  private function getSizeByFixedHeight($newHeight){
    $ratio = $this->width / $this->height;
    $newWidth = $newHeight * $ratio;
    return $newWidth;
  }

  private function getSizeByFixedWidth($newWidth){
    $ratio = $this->height / $this->width;
    $newHeight = $newWidth * $ratio;
    return $newHeight;
  }

  private function getSizeByAuto($newWidth, $newHeight){
    if ($this->height < $this->width)
    // *** Image to be resized is wider (landscape)
    {
        $optimalWidth = $newWidth;
        $optimalHeight= $this->getSizeByFixedWidth($newWidth);
    }
    elseif ($this->height > $this->width)
    // *** Image to be resized is taller (portrait)
    {
        $optimalWidth = $this->getSizeByFixedHeight($newHeight);
        $optimalHeight= $newHeight;
    }
    else
    // *** Image to be resizerd is a square
    {
        if ($newHeight < $newWidth) {
            $optimalWidth = $newWidth;
            $optimalHeight= $this->getSizeByFixedWidth($newWidth);
        } else if ($newHeight > $newWidth) {
            $optimalWidth = $this->getSizeByFixedHeight($newHeight);
            $optimalHeight= $newHeight;
        } else {
            // *** Sqaure being resized to a square
            $optimalWidth = $newWidth;
            $optimalHeight= $newHeight;
        }
    }

    return array('optimalWidth' => $optimalWidth, 'optimalHeight' => $optimalHeight);
  }

  private function getOptimalCrop($newWidth, $newHeight){

    $heightRatio = $this->height / $newHeight;
    $widthRatio  = $this->width /  $newWidth;

    if ($heightRatio < $widthRatio) {
        $optimalRatio = $heightRatio;
    } else {
        $optimalRatio = $widthRatio;
    }

    $optimalHeight = $this->height / $optimalRatio;
    $optimalWidth  = $this->width  / $optimalRatio;

    return array('optimalWidth' => $optimalWidth, 'optimalHeight' => $optimalHeight);
  }

  private function crop($optimalWidth, $optimalHeight, $newWidth, $newHeight){

    // *** Find center - this will be used for the crop
    $cropStartX = ( $optimalWidth / 2) - ( $newWidth /2 );
    $cropStartY = ( $optimalHeight/ 2) - ( $newHeight/2 );

    $crop = $this->imageResized;
    //imagedestroy($this->imageResized);

    // *** Now crop from center to exact requested size
    $this->imageResized = imagecreatetruecolor($newWidth , $newHeight);
    imagecopyresampled($this->imageResized, $crop , 0, 0, $cropStartX, $cropStartY, $newWidth, $newHeight , $newWidth, $newHeight);

  }

  public function saveImage($imageQuality=100, $savePath=''){

    if($savePath=='') $savePath=$this->filename;
    else{

      if($savePath!=$this->filename){

        $this->saveAltImage($imageQuality,$savePath);
        return true;
      }

    }

    switch($this->imageType) {
      case "image/gif":

        if (imagetypes() & IMG_GIF) {
          imagegif($this->imageResized, $savePath);
        }

      break;
      case "image/pjpeg":
      case "image/jpeg":
      case "image/jpg":

        if (imagetypes() & IMG_JPG) {
          imagejpeg($this->imageResized, $savePath, $imageQuality);
        }

      break;
      case "image/png":
      case "image/x-png":

        // *** Scale quality from 0-100 to 0-9
        $scaleQuality = round(($imageQuality/100) * 9);

        // *** Invert quality setting as 0 is best, not 9
        $invertScaleQuality = 9 - $scaleQuality;

        if (imagetypes() & IMG_PNG) {
            imagepng($this->imageResized, $savePath, $invertScaleQuality);
        }

      break;
      default:

      break;
    }

    imagedestroy($this->imageResized);

  }

  public function saveAltImage($imageQuality=100,$savePath){

    // *** Get extension
    $extension = strrchr($savePath, '.');
    $extension = strtolower($extension);

    switch($extension)
    {
        case '.jpg':
        case '.jpeg':
            if (imagetypes() & IMG_JPG) {
                imagejpeg($this->imageResized, $savePath, $imageQuality);
            }
            break;

        case '.gif':
            if (imagetypes() & IMG_GIF) {
                imagegif($this->imageResized, $savePath);
            }
            break;

        case '.png':
            // *** Scale quality from 0-100 to 0-9
            $scaleQuality = round(($imageQuality/100) * 9);

            // *** Invert quality setting as 0 is best, not 9
            $invertScaleQuality = 9 - $scaleQuality;

            if (imagetypes() & IMG_PNG) {
                imagepng($this->imageResized, $savePath, $invertScaleQuality);
            }
            break;

        // ... etc

        default:
            // *** No extension - No save.
            break;
    }

    imagedestroy($this->imageResized);
  }

}

?>
