<?php
/**
 *****************************************************************************
 * JBEI - SOFTWARE LICENSE (v1.0)
 *
 * You may not alter, transform, build upon, redistribute, share this work
 * without written permission from the author.
 ******************************************************************************
 * @category   Payment Gateway
 * @author     John Barounis
 * @copyright  Copyright (c)2018 https://www.johnbarounis.com
 ******************************************************************************
 */

class PayPal {

    var $code, $title, $description, $enabled, $sort_order, $form_action_url;

    public $MerchantReference = '';//filled later

    public $itemName = '';//filled later
    public $itemNumber = '';//filled later

    //public $_check = 0;

    public function __construct() {

        global $order;

        $this->code = 'PayPal';
        $this->title = MODULE_PAYMENT_PAYPAL_TEXT_TITLE;
        $this->description = MODULE_PAYMENT_PAYPAL_TEXT_DESCRIPTION;
        $this->enabled = (MODULE_PAYMENT_PAYPAL_STATUS == 'True') ? true : false;
        $this->sort_order = MODULE_PAYMENT_PAYPAL_SORTORDER;
        $this->form_action_url = MODULE_PAYMENT_PAYPAL_POSTURL;

        if ( (int)MODULE_PAYMENT_PAYPAL_ORDER_STATUS_ID > 0 ) {

            $this->order_status = MODULE_PAYMENT_PAYPAL_ORDER_STATUS_ID;

        }

        if ( is_object($order) ){

            $this->update_status();

        }

    }

    public function update_status() {

        global $order;

        if ( ($this->enabled == true) && ((int)MODULE_PAYMENT_PAYPAL_ZONE > 0) ) {

            $check_flag = false;

            $check_query = tep_db_query("select zone_id from " . TABLE_ZONES_TO_GEO_ZONES . " where geo_zone_id = '" . MODULE_PAYMENT_PAYPAL_ZONE . "' and zone_country_id = '" . $order->billing['country']['id'] . "' order by zone_id");

            while ($check = tep_db_fetch_array($check_query)) {

                if ($check['zone_id'] < 1) {

                  $check_flag = true;
                  break;

                } elseif ($check['zone_id'] == $order->billing['zone_id']) {

                  $check_flag = true;
                  break;

                }

            }

            if ( !$check_flag ) {

                $this->enabled = false;

            }
        }

    }

    public function javascript_validation() {

        return false;

    }

    public function selection() {

        return array('id' => $this->code, 'module' => $this->title);

    }

    public function pre_confirmation_check() {

        global $messageStack;

        if( isset( $_GET['Error'] ) ){

            $paypal_error_message = '';

            switch( $_GET['Error'] ){

                case 'failed_no_success':

                    $paypal_error_message = 'Λυπούμαστε! Η πληρωμή της παραγγελίας δεν ήταν επιτυχής.';

                break;

                case 'failed_data':

                    $paypal_error_message = 'Λυπούμαστε! Η πληρωμή της παραγγελίας δεν ήταν επιτυχής λόγω εσφαλμένων δεδομένων. Παρακαλώ επικοινωνήστε με τον διαχειριστή του '.STORE_NAME;

                break;

                case 'http_error':

                    $paypal_error_message = 'Λυπούμαστε! Η πληρωμή της παραγγελίας δεν ήταν επιτυχής λόγω εσφαλμένου πρωτοκόλλου επικοινωνίας. Παρακαλώ επικοινωνήστε με τον διαχειριστή του '.STORE_NAME;

                break;

                default:

                    $paypal_error_message = 'Λυπούμαστε! Η πληρωμή της παραγγελίας σας ακυρώθηκε.';

            }

            $messageStack->add('checkout_confirmation', $paypal_error_message);

        }


        if( isset( $_GET['tx'] ) ){

            $this->processFromPayPal();

        }

        return false;

    }

    public function processFromPayPal(){

        $pp_hostname = MODULE_PAYMENT_PAYPAL_HOSTNAME; // Change to www.sandbox.paypal.com to test against sandbox

        // read the post from PayPal system and add 'cmd'
        $req = 'cmd=_notify-synch';

        $tx_token = $_GET['tx'];

        $auth_token = MODULE_PAYMENT_PAYPAL_AUTHENTICATION_TOKEN;
        $req .= "&tx=$tx_token&at=$auth_token";

        $receiver_email = MODULE_PAYMENT_PAYPAL_BUSINESS_EMAIL;

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, MODULE_PAYMENT_PAYPAL_POSTURL);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER,1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $req);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
        //set cacert.pem verisign certificate path in curl using 'CURLOPT_CAINFO' field here,
        //if your server does not bundled with default verisign certificates.
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array("Host: $pp_hostname"));
        $res = curl_exec($ch);
        curl_close($ch);

        //var_dump($res);

        //die;

        if(!$res){
            //HTTP ERROR

            $data = null;

            $item_number = time();

            tep_db_query("insert into paypal_data ( MerchantReference, Timestamp, data, status ) VALUES ('".$item_number."', now() ,'".$data."', 'error http error') ");

            $payment_error_return = 'Error=http_error';
            tep_redirect(tep_href_link('checkout_confirmation.php', $payment_error_return, 'SSL', false, false));

        }else{

           //var_dump($res); die;

            // parse the data
            $lines = explode("\n", trim($res));
            $keyarray = array();
            if (strcmp ($lines[0], "SUCCESS") == 0) {
                for ($i = 1; $i < count($lines); $i++) {
                    $temp = explode("=", $lines[$i],2);
                    $keyarray[urldecode($temp[0])] = urldecode($temp[1]);
                }
                // check the payment_status is Completed
                // check that txn_id has not been previously processed
                // check that receiver_email is your Primary PayPal email
                // check that payment_amount/payment_currency are correct
                // process payment

                $error = true;

                $item_number = '';

                if( isset( $keyarray['receiver_email'] ) && $keyarray['receiver_email'] === $receiver_email ){

                    if( isset( $keyarray['payment_status'] ) && $keyarray['payment_status'] === 'Completed' ){

                        if( isset( $keyarray['item_number'] ) ){

                            $array = explode('#',$keyarray['item_number']);

                            $item_number = $array[0];
                            $customers_code = $array[1];

                            $data = json_encode($lines,JSON_UNESCAPED_UNICODE);

                            tep_db_query("insert into paypal_data ( customers_code, MerchantReference, Timestamp, data, status ) VALUES ('".$customers_code."','".$item_number."', now() ,'".$data."', 'success') on duplicate key update data = '".$data."', status = '".$status."', Timestamp = now() ");

                            $error = false;

                        }

                    }

                }

                if( $error ){

                    $payment_error_return = 'Error=failed_data';
                    tep_redirect(tep_href_link('checkout_confirmation.php', $payment_error_return, 'SSL', false, false));

                } else {

                    $returnparameters = 'MerchantReference=' . $item_number;

                    tep_redirect(tep_href_link('checkout_process.php', $returnparameters, 'SSL', true, false));

                }

            //echo '<pre>';
            //var_dump($keyarray);
            //echo '</pre>';
            } else if (strcmp ($lines[0], "FAIL") == 0) {

                $data = json_encode($lines,JSON_UNESCAPED_UNICODE);

                $item_number = time();

                tep_db_query("insert into paypal_data ( MerchantReference, Timestamp, data, status ) VALUES ('".$item_number."', now() ,'".$data."', 'error no success at payment when parsing data from paypal') ");

                $payment_error_return = 'Error=failed_no_success';
                tep_redirect(tep_href_link('checkout_confirmation.php', $payment_error_return, 'SSL', false, false));

            } else {

                $data = json_encode($lines,JSON_UNESCAPED_UNICODE);

                $item_number = time();

                tep_db_query("insert into paypal_data ( MerchantReference, Timestamp, data, status ) VALUES ('".$item_number."', now() ,'".$data."', 'error general from paypal') ");

                $payment_error_return = 'Error=failed_general';
                tep_redirect(tep_href_link('checkout_confirmation.php', $payment_error_return, 'SSL', false, false));

            }
        }

    }

    public function confirmation() {

        global $HTTP_POST_VARS, $order, $currencies, $currency, $customer_id;

        //tep_db_query("insert into paypal_data (orders_id, customers_code, MerchantReference, Timestamp, data, status) values ('0','0','". $this->MerchantReference ."', now() ,null ,null)");

        $this->itemName = 'Παραγγελία στο '.STORE_NAME;
        $this->itemNumber = time().'#'.$customer_id;

        $confirmation = array('title' => $this->title . ': ' . MODULE_PAYMENT_PAYPAL_WARNING);

        return $confirmation;

    }

    public function process_button() {

        global $HTTP_POST_VARS, $customer_id, $order, $currencies, $currency, $languages_id;

/*
    $language_query = tep_db_query("select code from " . TABLE_LANGUAGES . " where languages_id = '" . (int)$languages_id . "'");
    $piraeusbank_lang = tep_db_fetch_array($language_query);

    if(strtoupper($piraeusbank_lang['code']) == 'GR' || strtoupper($piraeusbank_lang['code']) == 'EL'){

      $languagecode = 'el-GR';

    } else {

      $languagecode = 'en-US';

    }
*/

        $set_currency = 'EUR';

        $Amount = number_format($order->info['total'] * $currencies->get_value($set_currency), '2', '.', '');

        $process_button_string =
        tep_draw_hidden_field('cmd', '_xclick') .
        tep_draw_hidden_field('charset', 'utf-8') .
        tep_draw_hidden_field('business', MODULE_PAYMENT_PAYPAL_BUSINESS_EMAIL) .
        tep_draw_hidden_field('item_name', $this->itemName) .
        tep_draw_hidden_field('item_number', $this->itemNumber) .
        tep_draw_hidden_field('currency_code', 'EUR') .
        tep_draw_hidden_field('return', MODULE_PAYMENT_PAYPAL_RETURNURL) .
        tep_draw_hidden_field('cancel_return', MODULE_PAYMENT_PAYPAL_CANCELRETURNURL) .
        tep_draw_hidden_field('amount', $Amount);
        //tep_draw_hidden_field ('Parameters', 'osCsid='.tep_session_id().'&MerchantReference='.$this->MerchantReference);
        //tep_draw_hidden_field ('Parameters', '');

        return $process_button_string;

    }

    public function before_process() {

        if( isset( $_GET['MerchantReference'] ) ){

            $this->MerchantReference = tep_db_prepare_input($_GET['MerchantReference']);

            $transtat_query = tep_db_query("select * from paypal_data where MerchantReference='".$this->MerchantReference."'");
            //$transtat = tep_db_fetch_array($transtat_query);

            if( tep_db_num_rows($transtat_query) <= 0 ){

                $messageStack->add('checkout_payment', 'Payment not processed. Please contact admin.');

                tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'error_message=' . urlencode('Error: Not found'), 'SSL', true, false));

            }

        } else {

            $messageStack->add('checkout_payment', 'Payment not processed. Please contact admin.');

            tep_redirect(tep_href_link(FILENAME_CHECKOUT_PAYMENT, 'error_message=' . urlencode('Error: MR Not found'), 'SSL', true, false));

        }

    }

    public function after_process() {

        global $HTTP_POST_VARS, $order, $insert_id;

        $MerchantReference = $this->MerchantReference;

        if( isset( $_GET['MerchantReference'] ) ){

            $MerchantReference = $_GET['MerchantReference'];

        }

        tep_db_perform('paypal_data', array(

            'orders_id' => (int)$insert_id,

        ), 'update', "MerchantReference = '" . $MerchantReference . "' ");

    }

    public function get_error($error='') {

        global $HTTP_GET_VARS;

        if ( $error == "" ) {

            $error = $HTTP_GET_VARS['error'];

        }

        $error = array(
            'title' => MODULE_PAYMENT_PAYPAL_TEXT_ERROR,
            'error' => stripslashes(urldecode($error))
        );

        return $error;

    }

    public function output_error() {

        return false;

    }

    public function check() {

        if (!isset($this->_check)) {

            $check_query = tep_db_query("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'MODULE_PAYMENT_PAYPAL_STATUS'");
            $this->_check = tep_db_num_rows($check_query);

        }

        return $this->_check;

    }

    public function install() {

        tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Enable PayPal Module', 'MODULE_PAYMENT_PAYPAL_STATUS', 'True', 'Do you want to accept PayPal payments?', '6', '0', 'tep_cfg_select_option(array(\'True\', \'False\'), ', now())");

        tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Business email', 'MODULE_PAYMENT_PAYPAL_BUSINESS_EMAIL', 'siagris-facilitator@hotmail.com', '', '6', '1', now())");

        tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Return Url - where to return user after successful payment', 'MODULE_PAYMENT_PAYPAL_RETURNURL', 'https://eshop1.homebath.gr/checkout_confirmation.php', '', '6', '2', now())");

        tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Cancel Return Url - where to return user after cancelled payment', 'MODULE_PAYMENT_PAYPAL_CANCELRETURNURL', 'https://eshop1.homebath.gr/checkout_confirmation.php?Error=1', '', '6', '3', now())");

        tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Post URL', 'MODULE_PAYMENT_PAYPAL_POSTURL', 'https://www.paypal.com/cgi-bin/webscr', '', '6', '4', now())");

        tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Hostname', 'MODULE_PAYMENT_PAYPAL_HOSTNAME', 'www.paypal.com', 'Change to www.sandbox.paypal.com to test against sandbox', '6', '5', now())");

        tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Authentication Token', 'MODULE_PAYMENT_PAYPAL_AUTHENTICATION_TOKEN', 'SbV9jn9GmCvti-4Wr7Cv3isGNvQm9dzgOGRIcZjVnYU-3iBOSsnHexSDQEy', '', '6', '6', now())");

        tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Unique sort order', 'MODULE_PAYMENT_PAYPAL_SORTORDER', '4', 'Sort order of display. Lowest is displayed first.', '6', '7', now())");

        tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, use_function, set_function, date_added) values ('Payment Zone', 'MODULE_PAYMENT_PAYPAL_ZONE', '0', 'If a zone is selected, only enable this payment method for that zone', '6', '12', 'tep_get_zone_class_title', 'tep_cfg_pull_down_zone_classes(', now())");

        tep_db_query("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) values ('Set Order Status', 'MODULE_PAYMENT_PAYPAL_ORDER_STATUS_ID', '0', 'Set the status of orders made with this payment module to this value', '6', '13', 'tep_cfg_pull_down_order_statuses(', 'tep_get_order_status_name', now())");

        tep_db_query("CREATE TABLE IF NOT EXISTS `paypal_data` (
                      `orders_id` int(11) UNSIGNED NOT NULL,
                      `customers_code` varchar(255) COLLATE utf8_unicode_ci DEFAULT NULL,
                      `MerchantReference` varchar(255) COLLATE utf8_unicode_ci NOT NULL,
                      `Timestamp` datetime DEFAULT NULL,
                      `data` longtext COLLATE utf8_unicode_ci,
                      `status` varchar(255) COLLATE utf8_unicode_ci DEFAULT NULL
                    ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;");

        tep_db_query("ALTER TABLE `paypal_data`
                          ADD PRIMARY KEY (`MerchantReference`),
                          ADD KEY `orders_id` (`orders_id`),
                          ADD KEY `customers_code` (`customers_code`);");

    }

    public function remove() {

        tep_db_query("delete from " . TABLE_CONFIGURATION . " where configuration_key in ('" . implode("', '", $this->keys()) . "')");

    }

    public function keys() {

        return array(
          'MODULE_PAYMENT_PAYPAL_STATUS',
          'MODULE_PAYMENT_PAYPAL_BUSINESS_EMAIL',
          'MODULE_PAYMENT_PAYPAL_RETURNURL',
          'MODULE_PAYMENT_PAYPAL_CANCELRETURNURL',
          'MODULE_PAYMENT_PAYPAL_POSTURL',
          'MODULE_PAYMENT_PAYPAL_HOSTNAME',
          'MODULE_PAYMENT_PAYPAL_AUTHENTICATION_TOKEN',
          'MODULE_PAYMENT_PAYPAL_SORTORDER',
          'MODULE_PAYMENT_PAYPAL_ZONE',
          'MODULE_PAYMENT_PAYPAL_ORDER_STATUS_ID',
        );

    }

}
